# -----------------------------------------------------------
# This script merges GTFS files from different agencies into a
# single GTFS zip
#
# Vincent Verbavatz - 2020
# -----------------------------------------------------------

import os, sys
from zipfile import ZipFile, BadZipfile
from osgeo import gdal, ogr, osr
from shutil import rmtree
import pandas as pd

def merge_txt(input_folder, files):

    file1 = files[0]
    output = os.path.basename(file1)
    data = ""

    data = pd.read_csv(file1)

    if 'trip_id' in list(data):
        data['trip_id'] = '1MERGE' + data['trip_id'].astype(str)
    if 'route_id' in list(data):
        data['route_id'] = '1MERGE' + data['route_id'].astype(str)
    if 'stop_id' in list(data):
        data['stop_id'] = '1MERGE' + data['stop_id'].astype(str)

    j = 2
    for file in files[1:]:
        try:
            data2 = pd.read_csv(file)

            if 'trip_id' in list(data):
                data2['trip_id'] = str(j) + 'MERGE' + data2['trip_id'].astype(str)
            if 'route_id' in list(data):
                data2['route_id'] = str(j) + 'MERGE' + data2['route_id'].astype(str)
            if 'stop_id' in list(data):
                data2['stop_id'] = str(j) + 'MERGE' + data2['stop_id'].astype(str)
            j += 1

            data = data.append(data2, sort=False)
        except FileNotFoundError:
            pass

    data.to_csv(input_folder + output, index=False)


if __name__ == '__main__':
    if len(sys.argv[1:]) == 2:
        #input1 = sys.argv[1]
        input_folder = sys.argv[1]
        output = sys.argv[2]
    else:
        print("Missing argument. Argument must be: <input_directory> <output.zip>")
        quit()

    inputs = []
    count = 0
    for file in os.listdir(input_folder):
        if file.endswith(".zip"):
            count += 1
            with ZipFile(input_folder + file, 'r') as zipObj:
                zipObj.extractall(input_folder + "Input" + str(count) + "/")
                inputs.append(input_folder + "Input" + str(count) + "/")

    for file in os.listdir([input_folder + folder for folder in os.listdir(input_folder)
    if os.path.isdir(input_folder + folder)][0]):
        merge_txt(input_folder, [input_folder + folder + "/" + file for folder in os.listdir(input_folder)
        if os.path.isdir(input_folder + folder)])


    with ZipFile(output,'w') as zip:
        for file in os.listdir(input_folder):
            if file.endswith(".txt"):
                zip.write(input_folder + file, file)

    for file in os.listdir(input_folder):
        if file.endswith(".txt"):
            os.remove(input_folder + file)
        if os.path.isdir(input_folder + file):
            if "Input" in file:
                rmtree(input_folder + file)
