# -----------------------------------------------------------
# This script reprojects country shapefiles into Mollweide
# coordinates.
#
# Vincent Verbavatz - 2020
# -----------------------------------------------------------

import os, sys
from osgeo import gdal, ogr, osr

#folder where country shapefiles are stored
folder_countries = 'Test1/'

#folder where reprojections are stored
mollweide_countries = 'Test2/'

#Defines mollweide projection from well known text
mollweide_wkt = 'PROJCS["World_Mollweide",GEOGCS["WGS 84",DATUM["WGS_1984", \
                SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]], \
                AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["Degree",0.0174532925199433]], \
                PROJECTION["Mollweide"],PARAMETER["central_meridian",0],PARAMETER["false_easting",0], \
                PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]], \
                AXIS["Easting",EAST],AXIS["Northing",NORTH]]'
mollweide = osr.SpatialReference()
mollweide.ImportFromWkt(mollweide_wkt)

drv = ogr.GetDriverByName("ESRI Shapefile")

#Defines Coordinate transformation
coordTrans = osr.CoordinateTransformation(inSpatialRef, mollweide)

nameField = ogr.FieldDefn('fuaname_en', ogr.OFTString)

for country in os.listdir(folder_countries):
    if country.endswith(".shp"):
        src_ds = ogr.Open(folder_countries + country, update = 0)

        country_layer = src_ds.GetLayer()

        #Checks input coordinates
        inSpatialRef = country_layer.GetSpatialRef()

        #Prepares mollweide output layer
        shp_ds = drv.CreateDataSource(mollweide_countries + country)
        mollweide_layer = shp_ds.CreateLayer(country, geom_type=ogr.wkbMultiPolygon, srs = mollweide)
        # add fields
        inLayerDefn = country_layer.GetLayerDefn()
        for i in range(0, inLayerDefn.GetFieldCount()):
            fieldDefn = inLayerDefn.GetFieldDefn(i)
            mollweide_layer.CreateField(fieldDefn)

        featureDefn = mollweide_layer.GetLayerDefn()

        i = 0
        for city in country_layer:
            polygon = city.GetGeometryRef()

            #converts to Mollweide
            polygon.Transform(coordTrans)

            outFeature = ogr.Feature(featureDefn)
            outFeature.SetGeometry(polygon)
            for i in range(0, featureDefn.GetFieldCount()):
                outFeature.SetField(featureDefn.GetFieldDefn(i).GetNameRef(), city.GetField(i))

            mollweide_layer.CreateFeature(outFeature)

            outFeature = None

        shp_ds.Destroy()
